﻿using System.Collections.ObjectModel;

using Prism.Commands;
using Prism.Mvvm;
using Prism.Regions;

using TestLibrary.Business;

namespace TestLibrary.ViewModels
{
    /// <summary>
    /// 사람 리스트 뷰 모델
    /// </summary>
    public class PersonListViewModel : BindableBase, INavigationAware
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// 영역 관리자 인터페이스
        /// </summary>
        private IRegionManager regionManager;

        /// <summary>
        /// 영역 탐색 저널 인터페이스
        /// </summary>
        private IRegionNavigationJournal journal;

        /// <summary>
        /// 사람 컬렉션
        /// </summary>
        private ObservableCollection<Person> personCollection;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Property
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 사람 컬렉션 - PersonCollection

        /// <summary>
        /// 사람 컬렉션
        /// </summary>
        public ObservableCollection<Person> PersonCollection
        {
            get
            {
                return this.personCollection;
            }
            set
            {
                SetProperty(ref this.personCollection, value);
            }
        }

        #endregion
        #region 사람 선택 명령 - SelectPersonCommand

        /// <summary>
        /// 사람 선택 명령
        /// </summary>
        public DelegateCommand<Person> SelectPersonCommand { get; private set; }

        #endregion
        #region 앞으로 가기 명령 - GoForwardCommand

        /// <summary>
        /// 앞으로 가기 명령
        /// </summary>
        public DelegateCommand GoForwardCommand { get; set; }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - PersonListViewModel(regionManager)

        /// <summary>
        /// 생성자
        /// </summary>
        /// <param name="regionManager">영역 관리자 인터페이스</param>
        public PersonListViewModel(IRegionManager regionManager)
        {
            this.regionManager = regionManager;

            SelectPersonCommand = new DelegateCommand<Person>(SelectPerson);

            ObservableCollection<Person> personCollection = new ObservableCollection<Person>();

            for(int i = 0; i < 10; i++)
            {
                personCollection.Add
                (
                    new Person()
                    {
                        FirstName = string.Format("First {0}", i),
                        LastName  = string.Format("Last {0}" , i),
                        Age       = i
                    }
                );
            }

            PersonCollection = personCollection;

            GoForwardCommand = new DelegateCommand(GoForward, CanGoForward);
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 탐색 타겟 여부 구하기 - IsNavigationTarget(navigationContext)

        /// <summary>
        /// 탐색 타겟 여부 구하기
        /// </summary>
        /// <param name="navigationContext">탐색 컨텍스트</param>
        /// <returns>탐색 타겟 여부</returns>
        public bool IsNavigationTarget(NavigationContext navigationContext)
        {
            return true;
        }

        #endregion
        #region 탐색 진입시 처리하기 - OnNavigatedTo(navigationContext)

        /// <summary>
        /// 탐색 진입시 처리하기
        /// </summary>
        /// <param name="navigationContext">탐색 컨텍스트</param>
        public void OnNavigatedTo(NavigationContext navigationContext)
        {
            this.journal = navigationContext.NavigationService.Journal;

            GoForwardCommand.RaiseCanExecuteChanged();
        }

        #endregion
        #region 탐색 이탈시 처리하기 - OnNavigatedFrom(navigationContext)

        /// <summary>
        /// 탐색 이탈시 처리하기
        /// </summary>
        /// <param name="navigationContext">탐색 컨텍스트</param>
        public void OnNavigatedFrom(NavigationContext navigationContext)
        {
        }

        #endregion

        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region 사람 선택하기 - SelectPerson(person)

        /// <summary>
        /// 사람 선택하기
        /// </summary>
        /// <param name="person">사람</param>
        private void SelectPerson(Person person)
        {
            NavigationParameters parameters = new NavigationParameters();

            parameters.Add("person", person);

            if(person != null)
            {
                this.regionManager.RequestNavigate("ContentRegion", "PersonView", parameters);
            }
        }

        #endregion
        #region 앞으로 가기 - GoForward()

        /// <summary>
        /// 앞으로 가기
        /// </summary>
        private void GoForward()
        {
            this.journal.GoForward();
        }

        #endregion
        #region 앞으로 가기 가능 여부 구하기 - CanGoForward()

        /// <summary>
        /// 앞으로 가기 가능 여부 구하기
        /// </summary>
        /// <returns>앞으로 가기 가능 여부</returns>
        private bool CanGoForward()
        {
            return this.journal != null && this.journal.CanGoForward;
        }

        #endregion
    }
}